/**
 * @function asController
 */
'use strict'

/** @lends asController */
function asController (instance, spec, context) {
  return Object.assign(
    {
      callbacks: {},
      controllerName: spec.name,
    },
    ...Object.keys(spec.methods).map((name) => ({
      [name]: async function methodProxy (...args) {
        const result = await instance[name](context, ...args)
        return result
      }.bind(this),
    })),
    {
      setCallback (name, callback) {
        let callbacks
        if (arguments.length >= 2) {
          callbacks = {[name]: callback}
        } else {
          callbacks = arguments[0] || {}
        }
        this.callbacks = {...this.callbacks, ...callbacks}
      },
      delCallback (...names) {
        for (const name of names) {
          delete this.callbacks[name]
        }
      },
    }
  )
}

module.exports = asController
